@@ -45,7 +45,7 @@ impl<'a> BytesStart<'a> {
     pub fn borrowed(content: &'a [u8], name_len: usize) -> Self {
         BytesStart {
             buf: Cow::Borrowed(content),
-            name_len: name_len,
+            name_len,
         }
     }
 
@@ -66,7 +66,7 @@ impl<'a> BytesStart<'a> {
     pub fn owned<C: Into<Vec<u8>>>(content: C, name_len: usize) -> BytesStart<'static> {
         BytesStart {
             buf: Cow::Owned(content.into()),
-            name_len: name_len,
+            name_len,
         }
     }
 
@@ -87,11 +87,44 @@ impl<'a> BytesStart<'a> {
         Self::owned(self.buf.into_owned(), self.name_len)
     }
 
-    /// Converts the event into an owned event without taking ownershiph of Event
+    /// Converts the event into an owned event without taking ownership of Event
     pub fn to_owned(&self) -> BytesStart<'static> {
         Self::owned(self.buf.to_owned(), self.name_len)
     }
 
+    /// Converts the event into a borrowed event. Most useful when paired with [`to_end`].
+    ///
+    /// # Example
+    ///
+    /// ```
+    /// # use quick_xml::{Error, Writer};
+    /// use quick_xml::events::{BytesStart, Event};
+    ///
+    /// struct SomeStruct<'a> {
+    ///     attrs: BytesStart<'a>,
+    ///     // ...
+    /// }
+    /// # impl<'a> SomeStruct<'a> {
+    /// # fn example(&self) -> Result<(), Error> {
+    /// # let mut writer = Writer::new(Vec::new());
+    ///
+    /// writer.write_event(Event::Start(self.attrs.to_borrowed()))?;
+    /// // ...
+    /// writer.write_event(Event::End(self.attrs.to_end()))?;
+    /// # Ok(())
+    /// # }}
+    /// ```
+    ///
+    /// [`to_end`]: #method.to_end
+    pub fn to_borrowed(&self) -> BytesStart {
+        BytesStart::borrowed(&self.buf, self.name_len)
+    }
+
+    /// Creates new paired close tag
+    pub fn to_end(&self) -> BytesEnd {
+        BytesEnd::borrowed(self.name())
+    }
+
     /// Consumes `self` and yield a new `BytesStart` with additional attributes from an iterator.
     ///
     /// The yielded items must be convertible to [`Attribute`] using `Into`.
@@ -140,6 +173,13 @@ impl<'a> BytesStart<'a> {
         Attributes::html(self, self.name_len)
     }
 
+    /// Gets the undecoded raw string with the attributes of this tag as a `&[u8]`,
+    /// including the whitespace after the tag name if there is any.
+    #[inline]
+    pub fn attributes_raw(&self) -> &[u8] {
+        &self.buf[self.name_len..]
+    }
+
     /// Add additional attributes to this tag using an iterator.
     ///
     /// The yielded items must be convertible to [`Attribute`] using `Into`.
@@ -166,10 +206,12 @@ impl<'a> BytesStart<'a> {
     ///
     /// [`unescaped()`]: #method.unescaped
     /// [`Reader::decode()`]: ../reader/struct.Reader.html#method.decode
-    #[cfg(feature = "encoding_rs")]
+    #[cfg(feature = "encoding")]
     #[inline]
     pub fn unescape_and_decode<B: BufRead>(&self, reader: &Reader<B>) -> Result<String> {
-        self.unescaped().map(|e| reader.decode(&*e).into_owned())
+        let decoded = reader.decode(&*self);
+        let unescaped = unescape(decoded.as_bytes()).map_err(Error::EscapeError)?;
+        String::from_utf8(unescaped.into_owned()).map_err(|e| Error::Utf8(e.utf8_error()))
     }
 
     /// Returns the unescaped and decoded string value.
@@ -182,11 +224,12 @@ impl<'a> BytesStart<'a> {
     ///
     /// [`unescaped()`]: #method.unescaped
     /// [`Reader::decode()`]: ../reader/struct.Reader.html#method.decode
-    #[cfg(not(feature = "encoding_rs"))]
+    #[cfg(not(feature = "encoding"))]
     #[inline]
     pub fn unescape_and_decode<B: BufRead>(&self, reader: &Reader<B>) -> Result<String> {
-        self.unescaped()
-            .and_then(|e| reader.decode(&*e).map(|s| s.to_owned()))
+        let decoded = reader.decode(&*self)?;
+        let unescaped = unescape(decoded.as_bytes()).map_err(Error::EscapeError)?;
+        String::from_utf8(unescaped.into_owned()).map_err(|e| Error::Utf8(e.utf8_error()))
     }
 
     /// Adds an attribute to this element.
@@ -462,6 +505,13 @@ impl<'a> BytesText<'a> {
         }
     }
 
+    /// Extracts the inner `Cow` from the `BytesText` event container.
+    #[cfg(feature = "serialize")]
+    #[inline]
+    pub(crate) fn into_inner(self) -> Cow<'a, [u8]> {
+        self.content
+    }
+
     /// gets escaped content
     ///
     /// Searches for '&' into content and try to escape the coded character if possible
@@ -471,14 +521,37 @@ impl<'a> BytesText<'a> {
     }
 
     /// helper method to unescape then decode self using the reader encoding
+    /// but without BOM (Byte order mark)
     ///
     /// for performance reasons (could avoid allocating a `String`),
     /// it might be wiser to manually use
     /// 1. BytesText::unescaped()
     /// 2. Reader::decode(...)
-    #[cfg(feature = "encoding_rs")]
-    pub fn unescape_and_decode<B: BufRead>(&self, reader: &Reader<B>) -> Result<String> {
-        self.unescaped().map(|e| reader.decode(&*e).into_owned())
+    #[cfg(feature = "encoding")]
+    pub fn unescape_and_decode_without_bom<B: BufRead>(
+        &self,
+        reader: &mut Reader<B>,
+    ) -> Result<String> {
+        let decoded = reader.decode_without_bom(&*self);
+        let unescaped = unescape(decoded.as_bytes()).map_err(Error::EscapeError)?;
+        String::from_utf8(unescaped.into_owned()).map_err(|e| Error::Utf8(e.utf8_error()))
+    }
+
+    /// helper method to unescape then decode self using the reader encoding
+    /// but without BOM (Byte order mark)
+    ///
+    /// for performance reasons (could avoid allocating a `String`),
+    /// it might be wiser to manually use
+    /// 1. BytesText::unescaped()
+    /// 2. Reader::decode(...)
+    #[cfg(not(feature = "encoding"))]
+    pub fn unescape_and_decode_without_bom<B: BufRead>(
+        &self,
+        reader: &Reader<B>,
+    ) -> Result<String> {
+        let decoded = reader.decode_without_bom(&*self)?;
+        let unescaped = unescape(decoded.as_bytes()).map_err(Error::EscapeError)?;
+        String::from_utf8(unescaped.into_owned()).map_err(|e| Error::Utf8(e.utf8_error()))
     }
 
     /// helper method to unescape then decode self using the reader encoding
@@ -487,10 +560,24 @@ impl<'a> BytesText<'a> {
     /// it might be wiser to manually use
     /// 1. BytesText::unescaped()
     /// 2. Reader::decode(...)
-    #[cfg(not(feature = "encoding_rs"))]
+    #[cfg(feature = "encoding")]
     pub fn unescape_and_decode<B: BufRead>(&self, reader: &Reader<B>) -> Result<String> {
-        self.unescaped()
-            .and_then(|e| reader.decode(&*e).map(|s| s.to_owned()))
+        let decoded = reader.decode(&*self);
+        let unescaped = unescape(decoded.as_bytes()).map_err(Error::EscapeError)?;
+        String::from_utf8(unescaped.into_owned()).map_err(|e| Error::Utf8(e.utf8_error()))
+    }
+
+    /// helper method to unescape then decode self using the reader encoding
+    ///
+    /// for performance reasons (could avoid allocating a `String`),
+    /// it might be wiser to manually use
+    /// 1. BytesText::unescaped()
+    /// 2. Reader::decode(...)
+    #[cfg(not(feature = "encoding"))]
+    pub fn unescape_and_decode<B: BufRead>(&self, reader: &Reader<B>) -> Result<String> {
+        let decoded = reader.decode(&*self)?;
+        let unescaped = unescape(decoded.as_bytes()).map_err(Error::EscapeError)?;
+        String::from_utf8(unescaped.into_owned()).map_err(|e| Error::Utf8(e.utf8_error()))
     }
 
     /// Gets escaped content.
@@ -660,8 +747,10 @@ mod test {
         let mut b = BytesStart::owned_name("test");
         assert_eq!(b.len(), 4);
         assert_eq!(b.name(), b"test");
+        assert_eq!(b.attributes_raw(), b"");
         b.push_attribute(("x", "a"));
         assert_eq!(b.len(), 10);
+        assert_eq!(b.attributes_raw(), b" x=\"a\"");
         b.set_name(b"g");
         assert_eq!(b.len(), 7);
         assert_eq!(b.name(), b"g");
