@@ -8,13 +8,14 @@ use std::path::Path;
 use std::str::from_utf8;
 
 #[cfg(feature = "encoding")]
-use encoding_rs::Encoding;
+use encoding_rs::{Encoding, UTF_16BE, UTF_16LE};
 
 use errors::{Error, Result};
 use events::{attributes::Attribute, BytesDecl, BytesEnd, BytesStart, BytesText, Event};
 
 use memchr;
 
+#[derive(Clone)]
 enum TagState {
     Opened,
     Closed,
@@ -61,6 +62,7 @@ enum TagState {
 ///     buf.clear();
 /// }
 /// ```
+#[derive(Clone)]
 pub struct Reader<B: BufRead> {
     /// reader
     reader: B,
@@ -88,13 +90,16 @@ pub struct Reader<B: BufRead> {
     #[cfg(feature = "encoding")]
     /// the encoding specified in the xml, defaults to utf8
     encoding: &'static Encoding,
+    #[cfg(feature = "encoding")]
+    /// check if quick-rs could find out the encoding
+    is_encoding_set: bool,
 }
 
 impl<B: BufRead> Reader<B> {
     /// Creates a `Reader` that reads from a reader implementing `BufRead`.
     pub fn from_reader(reader: B) -> Reader<B> {
         Reader {
-            reader: reader,
+            reader,
             opened_buffer: Vec::new(),
             opened_starts: Vec::new(),
             tag_state: TagState::Closed,
@@ -107,6 +112,8 @@ impl<B: BufRead> Reader<B> {
             ns_buffer: NamespaceBufferIndex::default(),
             #[cfg(feature = "encoding")]
             encoding: ::encoding_rs::UTF_8,
+            #[cfg(feature = "encoding")]
+            is_encoding_set: false,
         }
     }
 
@@ -194,12 +201,11 @@ impl<B: BufRead> Reader<B> {
     pub fn buffer_position(&self) -> usize {
         // when internal state is Opened, we have actually read until '<',
         // which we don't want to show
-        let offset = if let TagState::Opened = self.tag_state {
-            1
+        if let TagState::Opened = self.tag_state {
+            self.buf_position - 1
         } else {
-            0
-        };
-        self.buf_position - offset
+            self.buf_position
+        }
     }
 
     /// private function to read until '<' is found
@@ -207,10 +213,9 @@ impl<B: BufRead> Reader<B> {
     fn read_until_open<'a, 'b>(&'a mut self, buf: &'b mut Vec<u8>) -> Result<Event<'b>> {
         self.tag_state = TagState::Opened;
         let buf_start = buf.len();
-        match read_until(&mut self.reader, b'<', buf) {
+        match read_until(&mut self.reader, b'<', buf, &mut self.buf_position) {
             Ok(0) => Ok(Event::Eof),
-            Ok(n) => {
-                self.buf_position += n;
+            Ok(_) => {
                 let (start, len) = if self.trim_text {
                     match buf.iter().skip(buf_start).position(|&b| !is_whitespace(b)) {
                         Some(start) => (
@@ -250,31 +255,27 @@ impl<B: BufRead> Reader<B> {
         };
 
         if start != b'/' && start != b'!' && start != b'?' {
-            match read_elem_until(&mut self.reader, b'>', buf) {
+            match read_elem_until(&mut self.reader, b'>', buf, &mut self.buf_position) {
                 Ok(0) => Ok(Event::Eof),
-                Ok(n) => {
-                    self.buf_position += n;
+                Ok(_) => {
                     // we already *know* that we are in this case
                     self.read_start(&buf[buf_start..])
                 }
                 Err(e) => Err(e),
             }
         } else {
-            match read_until(&mut self.reader, b'>', buf) {
+            match read_until(&mut self.reader, b'>', buf, &mut self.buf_position) {
                 Ok(0) => Ok(Event::Eof),
-                Ok(n) => {
-                    self.buf_position += n;
-                    match start {
-                        b'/' => self.read_end(&buf[buf_start..]),
-                        b'!' => self.read_bang(buf_start, buf),
-                        b'?' => self.read_question_mark(&buf[buf_start..]),
-                        _ => unreachable!(
-                            "We checked that `start` must be one of [/!?], was {:?} \
+                Ok(_) => match start {
+                    b'/' => self.read_end(&buf[buf_start..]),
+                    b'!' => self.read_bang(buf_start, buf),
+                    b'?' => self.read_question_mark(&buf[buf_start..]),
+                    _ => unreachable!(
+                        "We checked that `start` must be one of [/!?], was {:?} \
                              instead.",
-                            start
-                        ),
-                    }
-                }
+                        start
+                    ),
+                },
                 Err(e) => Err(e),
             }
         }
@@ -284,7 +285,6 @@ impl<B: BufRead> Reader<B> {
     /// if `self.check_end_names`, checks that element matches last opened element
     /// return `End` event
     fn read_end<'a, 'b>(&'a mut self, buf: &'b [u8]) -> Result<Event<'b>> {
-        let len = buf.len();
         // XML standard permits whitespaces after the markup name in closing tags.
         // Let's strip them from the buffer before comparing tag names.
         let name = if self.trim_markup_names_in_closing_tags {
@@ -299,7 +299,7 @@ impl<B: BufRead> Reader<B> {
         };
         if self.check_end_names {
             let mismatch_err = |expected: &[u8], found: &[u8], buf_position: &mut usize| {
-                *buf_position -= len;
+                *buf_position -= buf.len();
                 Err(Error::EndEventMismatch {
                     expected: from_utf8(expected).unwrap_or("").to_owned(),
                     found: from_utf8(found).unwrap_or("").to_owned(),
@@ -332,65 +332,59 @@ impl<B: BufRead> Reader<B> {
         buf_start: usize,
         buf: &'b mut Vec<u8>,
     ) -> Result<Event<'b>> {
-        let len = buf.len();
-        if len >= buf_start + 3 && &buf[buf_start + 1..buf_start + 3] == b"--" {
-            let mut len = buf.len();
-            while (len - buf_start) < 5 || &buf[len - 2..] != b"--" {
+        if buf[buf_start..].starts_with(b"!--") {
+            while buf.len() < buf_start + 5 || !buf.ends_with(b"--") {
                 buf.push(b'>');
-                match read_until(&mut self.reader, b'>', buf) {
+                match read_until(&mut self.reader, b'>', buf, &mut self.buf_position) {
                     Ok(0) => {
-                        self.buf_position -= len;
+                        self.buf_position -= buf.len() - buf_start;
                         return Err(Error::UnexpectedEof("Comment".to_string()));
                     }
-                    Ok(n) => self.buf_position += n,
-                    Err(e) => return Err(e.into()),
+                    Ok(_) => (),
+                    Err(e) => return Err(e),
                 }
-                len = buf.len();
             }
+            let len = buf.len();
             if self.check_comments {
-                let mut offset = len - 3;
-                for w in buf[buf_start + 3..len - 1].windows(2) {
-                    if &*w == b"--" {
-                        self.buf_position -= offset;
-                        return Err(Error::UnexpectedToken("--".to_string()));
-                    }
-                    offset -= 1;
+                // search if '--' not in comments
+                if let Some(p) = memchr::memchr_iter(b'-', &buf[buf_start + 3..len - 2])
+                    .position(|p| buf[buf_start + 3 + p + 1] == b'-')
+                {
+                    self.buf_position -= buf.len() - buf_start + p;
+                    return Err(Error::UnexpectedToken("--".to_string()));
                 }
             }
             Ok(Event::Comment(BytesText::from_escaped(
                 &buf[buf_start + 3..len - 2],
             )))
-        } else if len >= buf_start + 8 {
+        } else if buf.len() >= buf_start + 8 {
             match &buf[buf_start + 1..buf_start + 8] {
                 b"[CDATA[" => {
-                    let mut len = buf.len();
-                    while len < 10 || &buf[len - 2..] != b"]]" {
+                    while buf.len() < 10 || !buf.ends_with(b"]]") {
                         buf.push(b'>');
-                        match read_until(&mut self.reader, b'>', buf) {
+                        match read_until(&mut self.reader, b'>', buf, &mut self.buf_position) {
                             Ok(0) => {
-                                self.buf_position -= len;
+                                self.buf_position -= buf.len() - buf_start;
                                 return Err(Error::UnexpectedEof("CData".to_string()));
                             }
-                            Ok(n) => self.buf_position += n,
+                            Ok(_) => (),
                             Err(e) => return Err(e),
                         }
-                        len = buf.len();
                     }
                     Ok(Event::CData(BytesText::from_escaped(
-                        &buf[buf_start + 8..len - 2],
+                        &buf[buf_start + 8..buf.len() - 2],
                     )))
                 }
                 b"DOCTYPE" => {
                     let mut count = buf.iter().skip(buf_start).filter(|&&b| b == b'<').count();
                     while count > 0 {
                         buf.push(b'>');
-                        match read_until(&mut self.reader, b'>', buf) {
+                        match read_until(&mut self.reader, b'>', buf, &mut self.buf_position) {
                             Ok(0) => {
-                                self.buf_position -= buf.len();
+                                self.buf_position -= buf.len() - buf_start;
                                 return Err(Error::UnexpectedEof("DOCTYPE".to_string()));
                             }
                             Ok(n) => {
-                                self.buf_position += n;
                                 let start = buf.len() - n;
                                 count += buf.iter().skip(start).filter(|&&b| b == b'<').count();
                                 count -= 1;
@@ -398,16 +392,15 @@ impl<B: BufRead> Reader<B> {
                             Err(e) => return Err(e),
                         }
                     }
-                    let len = buf.len();
                     Ok(Event::DocType(BytesText::from_escaped(
-                        &buf[buf_start + 8..len],
+                        &buf[buf_start + 8..buf.len()],
                     )))
                 }
-                _ => return Err(Error::UnexpectedBang),
+                _ => Err(Error::UnexpectedBang),
             }
         } else {
-            self.buf_position -= buf.len();
-            return Err(Error::UnexpectedBang);
+            self.buf_position -= buf.len() - buf_start;
+            Err(Error::UnexpectedBang)
         }
     }
 
@@ -422,6 +415,7 @@ impl<B: BufRead> Reader<B> {
                 // Try getting encoding from the declaration event
                 if let Some(enc) = event.encoder() {
                     self.encoding = enc;
+                    self.is_encoding_set = true;
                 }
                 Ok(Event::Decl(event))
             } else {
@@ -689,7 +683,52 @@ impl<B: BufRead> Reader<B> {
         self.encoding.decode(bytes).0
     }
 
-    /// Decodes a UTF8 slice regarless of XML declaration.
+    /// Decodes a UTF8 slice without BOM (Byte order mark) regardless of XML declaration.
+    ///
+    /// Decode `bytes` without BOM and with malformed sequences replaced with the
+    /// `U+FFFD REPLACEMENT CHARACTER`.
+    ///
+    /// # Note
+    ///
+    /// If you instead want to use XML declared encoding, use the `encoding` feature
+    #[inline]
+    #[cfg(not(feature = "encoding"))]
+    pub fn decode_without_bom<'c>(&self, bytes: &'c [u8]) -> Result<&'c str> {
+        if bytes.starts_with(b"\xEF\xBB\xBF") {
+            from_utf8(&bytes[3..]).map_err(Error::Utf8)
+        } else {
+            from_utf8(bytes).map_err(Error::Utf8)
+        }
+    }
+
+    /// Decodes a slice using without BOM (Byte order mark) the encoding specified in the XML declaration.
+    ///
+    /// Decode `bytes` without BOM and with malformed sequences replaced with the
+    /// `U+FFFD REPLACEMENT CHARACTER`.
+    ///
+    /// If no encoding is specified, defaults to UTF-8.
+    #[inline]
+    #[cfg(feature = "encoding")]
+    pub fn decode_without_bom<'b, 'c>(&'b mut self, mut bytes: &'c [u8]) -> Cow<'c, str> {
+        if self.is_encoding_set {
+            return self.encoding.decode_with_bom_removal(bytes).0;
+        }
+        if bytes.starts_with(b"\xEF\xBB\xBF") {
+            self.is_encoding_set = true;
+            bytes = &bytes[3..];
+        } else if bytes.starts_with(b"\xFF\xFE") {
+            self.is_encoding_set = true;
+            self.encoding = UTF_16LE;
+            bytes = &bytes[2..];
+        } else if bytes.starts_with(b"\xFE\xFF") {
+            self.is_encoding_set = true;
+            self.encoding = UTF_16BE;
+            bytes = &bytes[3..];
+        };
+        self.encoding.decode_without_bom_handling(bytes).0
+    }
+
+    /// Decodes a UTF8 slice regardless of XML declaration.
     ///
     /// Decode `bytes` with BOM sniffing and with malformed sequences replaced with the
     /// `U+FFFD REPLACEMENT CHARACTER`.
@@ -864,16 +903,24 @@ impl<'a> Reader<&'a [u8]> {
 /// read until `byte` is found or end of file
 /// return the position of byte
 #[inline]
-fn read_until<R: BufRead>(r: &mut R, byte: u8, buf: &mut Vec<u8>) -> Result<usize> {
+fn read_until<R: BufRead>(
+    r: &mut R,
+    byte: u8,
+    buf: &mut Vec<u8>,
+    position: &mut usize,
+) -> Result<usize> {
     let mut read = 0;
     let mut done = false;
     while !done {
         let used = {
             let available = match r.fill_buf() {
-                Ok(n) if n.is_empty() => return Ok(read),
+                Ok(n) if n.is_empty() => break,
                 Ok(n) => n,
                 Err(ref e) if e.kind() == io::ErrorKind::Interrupted => continue,
-                Err(e) => return Err(Error::Io(e)),
+                Err(e) => {
+                    *position += read;
+                    return Err(Error::Io(e));
+                }
             };
 
             match memchr::memchr(byte, available) {
@@ -891,6 +938,7 @@ fn read_until<R: BufRead>(r: &mut R, byte: u8, buf: &mut Vec<u8>) -> Result<usiz
         r.consume(used);
         read += used;
     }
+    *position += read;
     Ok(read)
 }
 
@@ -905,7 +953,12 @@ fn read_until<R: BufRead>(r: &mut R, byte: u8, buf: &mut Vec<u8>) -> Result<usiz
 /// (`Reference` is something like `&quot;`, but we don't care about escaped characters at this
 /// level)
 #[inline]
-fn read_elem_until<R: BufRead>(r: &mut R, end_byte: u8, buf: &mut Vec<u8>) -> Result<usize> {
+fn read_elem_until<R: BufRead>(
+    r: &mut R,
+    end_byte: u8,
+    buf: &mut Vec<u8>,
+    position: &mut usize,
+) -> Result<usize> {
     #[derive(Clone, Copy)]
     enum State {
         /// The initial state (inside element, but outside of attribute value)
@@ -924,7 +977,10 @@ fn read_elem_until<R: BufRead>(r: &mut R, end_byte: u8, buf: &mut Vec<u8>) -> Re
                 Ok(n) if n.is_empty() => return Ok(read),
                 Ok(n) => n,
                 Err(ref e) if e.kind() == io::ErrorKind::Interrupted => continue,
-                Err(e) => return Err(Error::Io(e)),
+                Err(e) => {
+                    *position += read;
+                    return Err(Error::Io(e));
+                }
             };
 
             let mut memiter = memchr::memchr3_iter(end_byte, b'\'', b'"', available);
@@ -962,6 +1018,7 @@ fn read_elem_until<R: BufRead>(r: &mut R, end_byte: u8, buf: &mut Vec<u8>) -> Re
         r.consume(used);
         read += used;
     }
+    *position += read;
     Ok(read)
 }
 
@@ -976,7 +1033,7 @@ pub(crate) fn is_whitespace(b: u8) -> bool {
 
 /// A namespace declaration. Can either bind a namespace to a prefix or define the current default
 /// namespace.
-#[derive(Debug)]
+#[derive(Debug, Clone)]
 struct Namespace {
     /// Index of the namespace in the buffer
     start: usize,
@@ -1026,7 +1083,7 @@ impl Namespace {
 /// A namespace management buffer.
 ///
 /// Holds all internal logic to push/pop namespaces with their levels.
-#[derive(Debug, Default)]
+#[derive(Debug, Default, Clone)]
 struct NamespaceBufferIndex {
     /// a buffer of namespace ranges
     slices: Vec<Namespace>,
@@ -1089,10 +1146,10 @@ impl NamespaceBufferIndex {
                             let start = buffer.len();
                             buffer.extend_from_slice(&*v);
                             self.slices.push(Namespace {
-                                start: start,
+                                start,
                                 prefix_len: 0,
                                 value_len: v.len(),
-                                level: level,
+                                level,
                             });
                         }
                         Some(&b':') => {
@@ -1100,10 +1157,10 @@ impl NamespaceBufferIndex {
                             buffer.extend_from_slice(&k[6..]);
                             buffer.extend_from_slice(&*v);
                             self.slices.push(Namespace {
-                                start: start,
+                                start,
                                 prefix_len: k.len() - 6,
                                 value_len: v.len(),
-                                level: level,
+                                level,
                             });
                         }
                         _ => break,
